/*
******************************************************************************
**

**  File        : LinkerScript.ld
**
**  Author		: Auto-generated by System Workbench for STM32
**
**  Abstract    : Linker script for STM32F746NGHx series
**                1024Kbytes FLASH and 320Kbytes RAM
**
**                Set heap size, stack size and stack location according
**                to application requirements.
**
**                Set memory bank area and size if external memory is used.
**
**  Target      : STMicroelectronics STM32
**
**  Distribution: The file is distributed “as is,” without any warranty
**                of any kind.
**
*****************************************************************************
** @attention
**
** <h2><center>&copy; COPYRIGHT(c) 2019 STMicroelectronics</center></h2>
**
** Redistribution and use in source and binary forms, with or without modification,
** are permitted provided that the following conditions are met:
**   1. Redistributions of source code must retain the above copyright notice,
**      this list of conditions and the following disclaimer.
**   2. Redistributions in binary form must reproduce the above copyright notice,
**      this list of conditions and the following disclaimer in the documentation
**      and/or other materials provided with the distribution.
**   3. Neither the name of STMicroelectronics nor the names of its contributors
**      may be used to endorse or promote products derived from this software
**      without specific prior written permission.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
** AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
** IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
** DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
** FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
** DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
** SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
** CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
** OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
** OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**
*****************************************************************************
*/


/* Entry Point */
ENTRY(Reset_Handler)

/* Generate a link error if heap and stack don't fit into RAM */
_Min_Heap_Size = 0x32000;       /* required amount of heap  */
_Min_Stack_Size = 0x400;        /* required amount of stack */
_Segger_RTT_Size = 4096 + 128;  /* Space allocated to Segger RTT logs + control block (allowing 128
                                 * bytes).  See SEGGER_RTT_Conf.h for buffer sizing.
                                 */
_FW_Magic_Reserved_Size = 8;    /* Space reserved at end of RAM to support
                                 * firmware upgrade operations.
                                 */

/* Specify the memory areas */
MEMORY
{
RAM (xrw)           : ORIGIN = 0x20000000, LENGTH = 320K
FLASH (rx)          : ORIGIN = 0x8010000, LENGTH = 960K
}

/* Define output sections */
SECTIONS
{
  /* The startup code goes first into FLASH */
  .isr_vector :
  {
    . = ALIGN(4);
    KEEP(*(.isr_vector)) /* Startup code */
    . = ALIGN(4);
  } >FLASH

  /* A trampoline which can be used to jump to the reset handler.  Useful as
   * an entry point at a known location for debugging when the main application
   * is not located at the start of flash. e.g. Due to normally being invoked
   * by a bootloader.
   */
  .reset_handler_trampoline :
   {
     . = ALIGN(4);
    /* Define a global symbol indicating location of the trampoline.  The CPU
     * program counter can be set to the value of this symbol to enter the
     * main application.
     */
    _reset_handler_trampoline = .;
    KEEP(*(.reset_handler_trampoline))
  } >FLASH

/*
 * The information block contains data fields with information
 * about the Firmware. This information can be used by both the
 * Bootloader and the Firmware image. It is placed in a fixed,
 * known location in Flash memory where the data can be accessed
 * internally by the Firmware and externally by the Bootloader.
 */
  .information_block :
  {
    . = ALIGN(4);
    KEEP(*(.information_block))	/* information block */
  } >FLASH

  /* The program code and other data goes into FLASH */
  .text :
  {
    . = ALIGN(4);
    *(.text)           /* .text sections (code) */
    *(.text*)          /* .text* sections (code) */
    *(.glue_7)         /* glue arm to thumb code */
    *(.glue_7t)        /* glue thumb to arm code */
    *(.eh_frame)

    KEEP (*(.init))
    KEEP (*(.fini))

    . = ALIGN(4);
    _etext = .;        /* define a global symbols at end of code */
  } >FLASH

  /* Constant data goes into FLASH */
  .rodata :
  {
    . = ALIGN(4);
    *(.rodata)         /* .rodata sections (constants, strings, etc.) */
    *(.rodata*)        /* .rodata* sections (constants, strings, etc.) */
    . = ALIGN(4);
  } >FLASH

  .ARM.extab   : { *(.ARM.extab* .gnu.linkonce.armextab.*) } >FLASH
  .ARM : {
    __exidx_start = .;
    *(.ARM.exidx*)
    __exidx_end = .;
  } >FLASH

  .preinit_array     :
  {
    PROVIDE_HIDDEN (__preinit_array_start = .);
    KEEP (*(.preinit_array*))
    PROVIDE_HIDDEN (__preinit_array_end = .);
  } >FLASH
  .init_array :
  {
    PROVIDE_HIDDEN (__init_array_start = .);
    KEEP (*(SORT(.init_array.*)))
    KEEP (*(.init_array*))
    PROVIDE_HIDDEN (__init_array_end = .);
  } >FLASH
  .fini_array :
  {
    PROVIDE_HIDDEN (__fini_array_start = .);
    KEEP (*(SORT(.fini_array.*)))
    KEEP (*(.fini_array*))
    PROVIDE_HIDDEN (__fini_array_end = .);
  } >FLASH

  /* used by the startup to initialize data */
  _sidata = LOADADDR(.data);

  /* Initialized data sections goes into RAM, load LMA copy after code */
  .data :
  {
    . = ALIGN(4);
    _sdata = .;        /* create a global symbol at data start */
    *(.data)           /* .data sections */
    *(.data*)          /* .data* sections */

    . = ALIGN(4);
    _edata = .;        /* define a global symbol at data end */
  } >RAM AT> FLASH

  /* Uninitialized data section */
  . = ALIGN(4);
  .bss :
  {
    /* This is used by the startup in order to initialize the .bss secion */
    _sbss = .;         /* define a global symbol at bss start */
    __bss_start__ = _sbss;

    *(.bss)
    *(.bss*)
    *(COMMON)

    . = ALIGN(4);
    _ebss = .;         /* define a global symbol at bss end */
    __bss_end__ = _ebss;
  } >RAM

  .heap (COPY):
  {
    . = ALIGN(8);
    PROVIDE(end = .);
    PROVIDE ( _end = . );
    KEEP(*(.heap*))
    . = . + _Min_Heap_Size;
    __heap_limit = .;
  } > RAM

  /** Variables in .noinit are not cleared or loaded on boot as per .bss/.data */
  .noinit (NOLOAD) :
  {
    _snoinit = .;
    KEEP(*(.noinit*))
    _enoinit = .;
    . = ALIGN(4);
  } >RAM

  /* Remainder of RAM after data/bss/noinit is in this order:
   * - Stack (as much space as is available until next section)
   * - Segger RTT log storage (fixed size allocation)
   * - Firmware update magic location (fixed size allocation)
   */

  /* .stack_dummy section doesn't contains any symbols. It is only
   * used for linker to calculate size of stack sections, and assign
   * values to stack symbols later */
  .stack_dummy (COPY):
  {
    KEEP(*(.stack*))
    . = . + _Min_Stack_Size;
    . = ALIGN(8);
  } > RAM

  _esegger_rtt_section = ORIGIN(RAM) + LENGTH(RAM) - _FW_Magic_Reserved_Size;
  _ssegger_rtt_section = _esegger_rtt_section - _Segger_RTT_Size;

  /* Top of stack (growing downwards) is just below fixed size Segger RTT logs */
  _estack = _ssegger_rtt_section;

  /* Lower stack limit based on minimum specified stack size */
  __stack_limit = _estack - SIZEOF(.stack_dummy);
  PROVIDE(__stack = _estack);

    _enoinit = .;
  /* Check if data + heap + stack exceeds RAM limit */
  ASSERT(__stack_limit >= _enoinit, "region RAM overflowed with stack")

  /* SEGGER RTT memory buffer.  The Segger RTT control block is located at a
   * known fixed location to ensure that the RTT buffers do not move addresses
   * between build.  Use 'nm <elf> | grep _ssegger_rtt$' to determine location
   * of RTT control block, then enter this number in 'Control block address'
   * in Segger tools.
   * Section marked NOLOAD to allow preservation of logs across reset, will not
   * be cleared with bss.
   */
  .segger_rtt _ssegger_rtt_section (NOLOAD) :
  {
    . = ALIGN(4);
    _ssegger_rtt = .;
    KEEP(*(.segger_rtt))
    _esegger_rtt = .;
  }

  /* Check if .segger_rtt section has overflowed allocated space for logs */
  ASSERT(_esegger_rtt <= _esegger_rtt_section, "region RAM overflowed with seggert RTT")

  /*
   * Reserve 8 bytes of SRAM at the top of memory to accommodate firmware update process,
   * Firmware update approach based on example given at
   * https://andrehessling.de/2015/02/16/howto-remanent-data-after-microcontroller-software-reset/
   */
  /* Reference to location for firmware update magic number, just after Segger
   * RTT log section.
   */
  _fw_update_magic_location = _esegger_rtt_section;

  /* Remove information from the standard libraries */
  /DISCARD/ :
  {
    libc.a ( * )
    libm.a ( * )
    libgcc.a ( * )
  }

  .ARM.attributes 0 : { *(.ARM.attributes) }
}
